"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Jsap_api, _Jsap_bench;
Object.defineProperty(exports, "__esModule", { value: true });
const defaults_1 = __importDefault(require("./defaults"));
const sepa_1 = __importDefault(require("./sepa"));
const querybench_1 = __importDefault(require("./querybench"));
function trasformBindings(bindings = {}, forcedBindings = {}) {
    let result = {};
    Object.keys(bindings).forEach(k => {
        result[k] = {
            value: bindings[k],
            type: forcedBindings[k] ? forcedBindings[k].type : "literal"
        };
    });
    result = Object.assign(forcedBindings, result);
    return result;
}
class Jsap {
    constructor(config = {}) {
        _Jsap_api.set(this, void 0);
        _Jsap_bench.set(this, void 0);
        if (typeof config === 'string') {
            // TODO: validate the parsed JSON
            config = JSON.parse(config);
        }
        let parameters = (({ host, sparql11protocol, sparql11seprotocol }) => (prune({ host, sparql11protocol, sparql11seprotocol })))(config);
        parameters = Object.assign({}, defaults_1.default, parameters);
        Object.assign(this, parameters);
        this.namespaces = config.namespaces ? config.namespaces : {};
        this.extended = config.extended ? config.extended : {};
        this.updates = config.updates ? config.updates : {};
        this.queries = config.queries ? config.queries : {};
        __classPrivateFieldSet(this, _Jsap_api, new sepa_1.default(parameters), "f");
        __classPrivateFieldSet(this, _Jsap_bench, new querybench_1.default(this.namespaces), "f");
        Object.keys(this.updates).forEach(k => {
            this[k] = binds => {
                return this.update(k, binds);
            };
        });
        Object.keys(this.queries).forEach(k => {
            if (this[k]) {
                delete (this[k]);
            }
            else {
                this[k] = (binds) => {
                    return this.subscribe(k, binds);
                };
                this[k].query = binds => {
                    return this.query(k, binds);
                };
            }
        });
    }
    query(key, bindings) {
        let query = this.queries[key].sparql;
        let binds = trasformBindings(bindings, this.queries[key].forcedBindings);
        let config = (({ host, sparql11seprotocol }) => (prune({ host, sparql11seprotocol })))(this.queries[key]);
        query = __classPrivateFieldGet(this, _Jsap_bench, "f").sparql(query, binds);
        return __classPrivateFieldGet(this, _Jsap_api, "f").query(query, config);
    }
    subscribe(key, bindings) {
        let query = this.queries[key].sparql;
        let binds = trasformBindings(bindings, this.queries[key].forcedBindings);
        let config = (({ host, sparql11seprotocol }) => (prune({ host, sparql11seprotocol })))(this.queries[key]);
        query = __classPrivateFieldGet(this, _Jsap_bench, "f").sparql(query, binds);
        return __classPrivateFieldGet(this, _Jsap_api, "f").subscribe(query, config);
    }
    update(key, bindings) {
        let update = this.updates[key].sparql;
        let binds = trasformBindings(bindings, this.updates[key].forcedBindings);
        let config = (({ host, sparql11protocol }) => (prune({ host, sparql11protocol })))(this.updates[key]);
        update = __classPrivateFieldGet(this, _Jsap_bench, "f").sparql(update, binds);
        return __classPrivateFieldGet(this, _Jsap_api, "f").update(update, config);
    }
    producer(key) {
        return binds => { this.update(key, binds); };
    }
    consumer(key) {
        return binds => { this.subscribe(key, binds); };
    }
    get Producers() {
        let result = {};
        Object.keys(this.updates).forEach(k => {
            result[k] = binds => {
                return this.update(k, binds);
            };
        });
        return result;
    }
    get Consumers() {
        let result = {};
        Object.keys(this.queries).forEach(k => {
            result[k] = (binds) => { return this.subscribe(k, binds); };
        });
        return result;
    }
}
_Jsap_api = new WeakMap(), _Jsap_bench = new WeakMap();
/**
 * Removes undedfined properties
 * @param {Object} obj
 */
function prune(obj) {
    for (const key in obj) {
        if (obj.hasOwnProperty(key)) {
            const element = obj[key];
            if (!element) {
                delete (obj[key]);
            }
        }
    }
    return obj;
}
exports.default = Jsap;
//# sourceMappingURL=jsap.js.map